`define ZERO        0
`define	BIT_WIDTH   24
`define LOOP_OUT    (`BIT_WIDTH + 1)
`define MSB         (`BIT_WIDTH - 1)

module Amanero_to_PCM (
    input wire left_data,	// may be I2S data
    input wire right_data,	// may be I2S LRCK
    input wire ds_clock,	// may be I2S BCK
	input wire pcm_off,
    output reg signed [7:0] left_out,
    output reg signed [7:0] right_out
);

// I2S -> DeltaSigma section
reg signed [`MSB:0] pcm_in;
reg signed [`MSB:0] pcm_temp;
reg signed [`MSB:0] left_memory;
reg signed [`MSB:0] right_memory;
reg signed [`BIT_WIDTH:0] left_sum;
reg signed [`BIT_WIDTH:0] right_sum;
reg [4:0] in_count;
reg lrck_memory;

initial begin
    // initialize DeltaSigma modulator
    pcm_in <= `ZERO;
    pcm_temp <= `ZERO;
    left_memory <= `ZERO;
    right_memory <= `ZERO;
    left_sum <= `ZERO;
    right_sum <= `ZERO;
    left_out <= `ZERO;
    right_out <= `ZERO;
    in_count <= `LOOP_OUT;
    lrck_memory <= 1;
end

always @(posedge ds_clock or posedge pcm_off) begin
	if (pcm_off) begin
		pcm_in <= `ZERO;
		pcm_temp <= `ZERO;
		left_memory <= `ZERO;
		right_memory <= `ZERO;
		in_count <= `LOOP_OUT;
		lrck_memory <= 1;
		left_out <= `ZERO;
		right_out <= `ZERO;
	end else begin
        // PCM 24bit integer read from I2S
        lrck_memory <= right_data;
        // I2S -> DeltaSigma -> PDM
		if (right_data != lrck_memory) begin
			in_count <= 5'b00000;
			if (lrck_memory) begin
				left_memory <= pcm_temp;
				right_memory <= pcm_in - {(pcm_in[23] ? 7'h7f : 7'h00), pcm_in[23:7]};
			end else pcm_temp <= pcm_in - {(pcm_in[23] ? 7'h7f : 7'h00), pcm_in[23:7]};
		end else if (in_count < `BIT_WIDTH) begin
			in_count <= in_count + 1;
			pcm_in <= {pcm_in[(`MSB - 1):0], left_data};
	    end
        if (left_sum[24] != left_sum[23])
                left_out <= left_sum[24] ? 8'h80 : 8'h7f;
        else
                left_out <= left_sum[23:16];
        if (right_sum[24] != right_sum[23])
                right_out <= right_sum[24] ? 8'h80 : 8'h7f;
        else
                right_out <= right_sum[23:16];
	end
end

always @(negedge ds_clock or posedge pcm_off) begin
	if (pcm_off) begin
		left_sum <= `ZERO;
		right_sum <= `ZERO;
	end else begin
        if (left_sum[24] != left_sum[23])
                left_sum <= {left_memory[23], left_memory};
        else
                left_sum <= {(left_sum[15] ? 10'b0000000000 : 10'b1111111111), left_sum[14:0]} + {left_memory[23], left_memory};
        if (right_sum[24] != right_sum[23])
                right_sum <= {right_memory[23], right_memory};
        else
                right_sum <= {(right_sum[15] ? 10'b0000000000 : 10'b1111111111), right_sum[14:0]} + {right_memory[23], right_memory};
	end
end

endmodule
