module DiscreteDAC (
        // inputs
        input   wire        I2S_DATA,   // I2S data
        input   wire        I2S_LRCK,   // I2S LRCK
        input   wire        I2S_BCLK,   // I2S BCLK
        input   wire        DSD_ON,     // Amanero DSD mode
        input   wire        MUTE,       // Amanero Mute
        
        // outputs
		output	wire  [7:0] LED,
        output  wire  [7:0] OUT_L,      // Left Channel
        output  wire  [7:0] OUT_R       // Right Channel
        );


//----------------------------------------------------------------------------
//                                                                          --
//                       ARCHITECTURE DEFINITION                            --
//                                                                          --
//----------------------------------------------------------------------------
//------------------------------
// INTERNAL SIGNAL DECLARATIONS: 
//------------------------------
// parameters (constants)

// wires (assigns)

wire signed [7:0] inner_left;
wire signed [7:0] inner_right;
wire	PCM_OFF;

wire signed [7:0] left_signal;
wire signed [7:0] right_signal;

// regs (always)
reg [21:0] bclk_count;
reg [15:0] lrck_count;
reg [15:0] data_count;

//-------------------------------------//
//-- assign (non-process) operations --//
//-------------------------------------//

assign PCM_OFF = DSD_ON | MUTE;
assign OUT_L[7] = (DSD_ON ? left_signal[7] : ~inner_left[7]);
assign OUT_L[6] = (DSD_ON ? left_signal[6] : inner_left[6]);
assign OUT_L[5] = (DSD_ON ? left_signal[5] : inner_left[5]);
assign OUT_L[4] = (DSD_ON ? left_signal[4] : inner_left[4]);
assign OUT_L[3] = (DSD_ON ? left_signal[3] : inner_left[3]);
assign OUT_L[2] = (DSD_ON ? left_signal[2] : inner_left[2]);
assign OUT_L[1] = (DSD_ON ? left_signal[1] : inner_left[1]);
assign OUT_L[0] = (DSD_ON ? left_signal[0] : inner_left[0]);
assign OUT_R[7] = (DSD_ON ? right_signal[7] : ~inner_right[7]);
assign OUT_R[6] = (DSD_ON ? right_signal[6] : inner_right[6]);
assign OUT_R[5] = (DSD_ON ? right_signal[5] : inner_right[5]);
assign OUT_R[4] = (DSD_ON ? right_signal[4] : inner_right[4]);
assign OUT_R[3] = (DSD_ON ? right_signal[3] : inner_right[3]);
assign OUT_R[2] = (DSD_ON ? right_signal[2] : inner_right[2]);
assign OUT_R[1] = (DSD_ON ? right_signal[1] : inner_right[1]);
assign OUT_R[0] = (DSD_ON ? right_signal[0] : inner_right[0]);

assign LED[7] = data_count[15];
assign LED[6] = data_count[15];
assign LED[5] = 1;
assign LED[4] = lrck_count[15];
assign LED[3] = lrck_count[15];
assign LED[2] = 1;
assign LED[1] = bclk_count[21];
assign LED[0] = bclk_count[21];

Amanero_to_PCM	TO_PCM(I2S_DATA, I2S_LRCK, I2S_BCLK, PCM_OFF, inner_left, inner_right);
DS_31STATE	DS_31STATE_L(I2S_DATA, I2S_BCLK, (~DSD_ON | MUTE), left_signal);
DS_31STATE	DS_31STATE_R(I2S_LRCK, I2S_BCLK, (~DSD_ON | MUTE), right_signal);

always @(posedge I2S_DATA) data_count = data_count + 16'h0001;
always @(posedge I2S_LRCK) lrck_count = lrck_count + 16'h0001;
always @(posedge I2S_BCLK) bclk_count = bclk_count + 22'h000001;
endmodule
